package com.lokesh.sse.controller;


import com.lokesh.sse.model.LiveScore;
import com.lokesh.sse.service.LiveScoreHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.codec.ServerSentEvent;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.time.Duration;

@RestController
@RequestMapping("/api/v1")
public class LiveScoreController {
    private static final Logger LOGGER = LoggerFactory.getLogger(LiveScoreController.class);

    private final LiveScoreHandler processor;

    public LiveScoreController(LiveScoreHandler processor) {
        this.processor = processor;
    }

    @PostMapping("/live-scores")
    @ResponseStatus(HttpStatus.CREATED)
    public Mono<LiveScore> sender(@RequestBody LiveScore liveScore) {
        LOGGER.info("Received '{}'", liveScore);
        processor.publish(liveScore);
        return Mono.just(liveScore);
    }

    @GetMapping(path = "/live-scores", produces = MediaType.TEXT_EVENT_STREAM_VALUE)
    public Flux<ServerSentEvent<Object>> consumer() {
        return Flux.create(sink -> processor.subscribe(sink::next))
                .map(liveScore -> ServerSentEvent.builder()
                        .id("id")
                        .data(liveScore)
                        .comment("hi guys!! this is real time broadcasting you")
                        .event("goal")
                        .retry(Duration.ofMillis(200))
                        .build());
    }
}
